package com.example.iport

import android.Manifest
import android.content.Context
import android.content.pm.PackageInfo
import android.content.pm.PackageManager
import android.os.Build
import androidx.core.content.ContextCompat
import com.phidget22.*
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.FlutterPlugin.FlutterPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.util.*


// Copyright 2017 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
/** RFIDPlugin  */
class RFIDPlugin : MethodCallHandler, FlutterPlugin {
    private var applicationContext: Context? = null
    private var methodChannel: MethodChannel? = null
    private var rfid : RFID? = null
    private var packageManager : PackageManager? = null

    override fun onAttachedToEngine(binding: FlutterPluginBinding) {
        packageManager = binding.applicationContext.packageManager

        onAttachedToEngine(
                binding.applicationContext, binding.flutterEngine.dartExecutor)
    }

    private fun onAttachedToEngine(applicationContext: Context, messenger: BinaryMessenger) {
        this.applicationContext = applicationContext
        packageManager = applicationContext.packageManager

        methodChannel = MethodChannel(messenger, "com.example.iport.rfid")
        methodChannel!!.setMethodCallHandler(this)
    }

    override fun onDetachedFromEngine(binding: FlutterPluginBinding) {
        applicationContext = null
        methodChannel!!.setMethodCallHandler(null)
        methodChannel = null
    }

    private fun setupPermissions() {
        val permission = ContextCompat.checkSelfPermission(applicationContext!!, Manifest.permission.UPDATE_DEVICE_STATS)

        /*
            if (packageManager!!.hasSystemFeature(PackageManager.FEATURE_USB_HOST)){
                methodChannel!!.invokeMethod("initRFID hata:","Erişim Hatası")
            }
        */
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        setupPermissions()

        try {
            if (call.method == "getAll") {
                val pm = packageManager!!
                val info = pm.getPackageInfo(applicationContext!!.packageName, 0)
                val map: MutableMap<String, String> = HashMap()
                map["appName"] = info.applicationInfo.loadLabel(pm).toString()
                map["packageName"] = applicationContext!!.packageName
                map["version"] = info.versionName
                map["buildNumber"] = RFIDPlugin.Companion.getLongVersionCode(info).toString()
                result.success(RFIDPlugin.Companion.getLongVersionCode(info).toString())

            }
            else if(call.method == "initAndConnect") {
                try {
                    if(rfid == null)
                    {
                        rfid  = RFID()
                        if (packageManager!!.hasSystemFeature(PackageManager.FEATURE_USB_HOST))
                            com.phidget22.usb.Manager.Initialize(applicationContext)

                        attachEvent(result)
                    }


                    // result.success("RFID nesnesi oluşturuldu")
                }
                catch (ex: Exception){
                    methodChannel!!.invokeMethod("initRFID hata 0:",  ex.message)

                    result.success("RFID nesnesi hata 0 : " + ex.message)

                }
                catch (ex: java.lang.Exception){
                    methodChannel!!.invokeMethod("initRFID hata 1:",  ex.message)

                    result.success("RFID nesnesi hata 1 : " + ex.message)

                }
                catch (ex: PhidgetException){
                    methodChannel!!.invokeMethod("initRFID hata 2:",  ex.message)

                    result.success("RFID nesnesi hata 2 : " + ex.message)
                }

            }
            else if(call.method == "initRFID") {
                try {
                    rfid  = RFID()
                    result.success("RFID nesnesi oluşturuldu")
                }
                catch (ex: Exception){
                    methodChannel!!.invokeMethod("initRFID hata 0:",  ex.message)

                    result.success("RFID nesnesi hata 0 : " + ex.message)

                }
                catch (ex: java.lang.Exception){
                    methodChannel!!.invokeMethod("initRFID hata 1:",  ex.message)

                    result.success("RFID nesnesi hata 1 : " + ex.message)

                }
                catch (ex: PhidgetException){
                    methodChannel!!.invokeMethod("initRFID hata 2:",  ex.message)

                    result.success("RFID nesnesi hata 2 : " + ex.message)
                }

            }
            else if(call.method == "checkConnection") {

                try {
                    // com.phidget22.usb.Manager.Initialize(applicationContext)
                    //Allow direct USB connection of Phidgets
                    if (packageManager!!.hasSystemFeature(PackageManager.FEATURE_USB_HOST))
                        com.phidget22.usb.Manager.Initialize(applicationContext)

                    //Enable server discovery to list remote Phidgets
                    // applicationContext!!.getSystemService((Context.NSD_SERVICE)
                    // Net.enableServerDiscovery(ServerType.DEVICE_REMOTE)

                    result.success("RFID kontroller başarılı")

                }
                catch (ph : PhidgetException)
                {
                    methodChannel!!.invokeMethod("checkConnection hata:",  ph.message)

                    result.success("RFID kontroller Hatalı : " + ph.detail)
                }

            }
            else if(call.method == "getLibraryVersion") {

                try {

                    result.success(Phidget.getLibraryVersion())

                }
                catch (ph : PhidgetException)
                {
                    methodChannel!!.invokeMethod("getLibraryVersion hata:",  ph.message)

                    result.success("RFID getLibraryVersion Hatalı : " + ph.detail)
                }

            }
            else if(call.method == "getAttached") {

                try {
                    val Attached = rfid!!.attached

                    result.success(Attached)

                }
                catch (ph : PhidgetException)
                {
                    methodChannel!!.invokeMethod("getAttached hata:",  ph.message)
                    result.success(false)
                }

            }
            else if(call.method == "getLastTag") {

                try {

                    if(rfid!!.attached == false)
                        result.success("")
                    else {

                        if (rfid!!.tagPresent == true) {
                            val lastTag = rfid!!.lastTag
                            result.success(lastTag.tagString)
                        } else
                            result.success("")
                    }

                }
                catch (ph : PhidgetException)
                {
                    methodChannel!!.invokeMethod("getLastTag hata:",  ph.message)
                    result.success("getLastTag hata: " +  ph.message)

                }

            }
            else if(call.method == "attachEvents") {
                attachEvent(result)
            }
            else {
                result.notImplemented()
            }
        } catch (ex: Exception) {
            methodChannel!!.invokeMethod("UnHandled hata:",  ex.message)

            result.error("UnHandled Erro : ", ex.message, null)
        } catch (ex: PackageManager.NameNotFoundException) {
            methodChannel!!.invokeMethod("UnHandled hata 2:",  ex.message)

            result.error("Name not found", ex.message, null)
        }
    }

    private fun attachEvent(result: MethodChannel.Result) {
        try {
            methodChannel!!.invokeMethod("attached :",  rfid!!.attached)

            rfid!!.addAttachListener { AttachEventHandler(rfid!!) }
            rfid!!.addDetachListener { DetachEventHandler(rfid!!) }
            rfid!!.addErrorListener { errorEvent -> ErrorEventHandler(rfid!!, errorEvent) }
            rfid!!.addTagListener { tagEvent -> RFIDTagEventHandler(rfid!!, tagEvent) }
            rfid!!.addTagLostListener { tagLostEvent -> RFIDTagLostEventHandler(rfid!!, tagLostEvent) }
            rfid!!.open()

            if(rfid!!.attached) {
                rfid!!.antennaEnabled = true
                result.success(true)
            }else
                result.success(true)

        }
        catch (ph : PhidgetException)
        {
            methodChannel!!.invokeMethod("attachEvents hata:",  ph.message)

            result.success(false)
        }
    }




    companion object {
        /** Plugin registration.  */
        fun registerWith(registrar: Registrar) {
            val instance = RFIDPlugin()
            instance.onAttachedToEngine(registrar.context(), registrar.messenger())
        }

        private fun getLongVersionCode(info: PackageInfo): Long {
            return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
                info.longVersionCode
            } else info.versionCode.toLong()
        }
    }


    internal inner class AttachEventHandler(var ch: Phidget) : Runnable {

        override fun run() {
            try {
                rfid!!.antennaEnabled = true
                methodChannel!!.invokeMethod("AttachEventHandler", ch.getDeviceName())

                /*
                    nameTxt.setText(ch.getDeviceName())
                    serialTxt.setText(Integer.toString(ch.getDeviceSerialNumber()))
                    versionTxt.setText(Integer.toString(ch.getDeviceVersion()))
                    channelTxt.setText(Integer.toString(ch.getChannel()))
                    hubPortTxt.setText(Integer.toString(ch.getHubPort()))
                    labelTxt.setText(ch.getDeviceLabel())

                */
            } catch (e: PhidgetException) {
                e.printStackTrace()
            }

            //notify that we're done
            synchronized(this) {

                print("")
                //this.notify()
            }
        }
    }

    internal inner class DetachEventHandler(var ch: Phidget) : Runnable {

        override fun run() {

            methodChannel!!.invokeMethod("DetachEventHandler", ch.getDeviceName())

            //notify that we're done
            synchronized(this) {
                print("")
            }
        }
    }

    internal inner class ErrorEventHandler(var ch: Phidget, var errorEvent: ErrorEvent) : Runnable {

        override fun run() {
            methodChannel!!.invokeMethod("ErrorEventHandler", errorEvent.getDescription())
        }
    }

    internal inner class RFIDTagEventHandler(var ch: Phidget, var tagEvent: RFIDTagEvent) : Runnable {

        override fun run() {
            var ss = tagEvent.getTag() + "-" + tagEvent.getProtocol().getMessage()

            methodChannel!!.invokeMethod("RFIDTagEventHandler", ss)

        }
    }

    internal inner class RFIDTagLostEventHandler(var ch: Phidget, var tagLostEvent: RFIDTagLostEvent) : Runnable {


        override fun run() {
            methodChannel!!.invokeMethod("RFIDTagLostEventHandler", ch.getDeviceName())

        }
    }
}